import { EncryptionFunction, DecryptionFunction, GetUserInfoFunction, CheckNetWorkFunction } from '../native/JSBridge.js';
import { GUID } from "../../utils/commonUtil";
import axios from "axios";
import Qs from "qs";
import { stroageData, getDataFromStroageData } from './demoStroage.js';
import ERROR_MAP from './errorMap';
axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded';
axios.defaults.timeout = 90000; //请求超时时间
axios.defaults.transformRequest = [function (data) {
    // 做任何你想要的数据转换
    return Qs.stringify(data);
}];

//添加响应拦截器
axios.interceptors.response.use(function (response) {
    //对响应数据做些事
    return response.data;
}, function (error) {
    //请求错误时做些事
    return Promise.reject(error);
});

var DemoConfig = {
    demoMicAppName: "", //根据服务名称设置缓存数据key值
    noCaheList: [], // 不需要缓存的请求, 自己按照应用往里面加
    specialUrl: function specialUrl() {}, //特殊处理的url，都在这里了需要不同的key值 如：如果是审批列表，需要通过参数type区分 我的审批、我的申请、知会我
    isCatchData: false, // 是否抓数据模式， 发布时必须是false, 只有抓demo数据时置为true
    demoData: ""
};

/**
 * post请求(data加密)
 * @param url
 * @param data
 */
function doPostRequest(url) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var p = new Promise(function (res, rej) {
        var cryptKey = "";
        var zip = 1;
        var keyType = 24;

        // start mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
        // 如果是demo环境，直接从缓存取数据返回
        if (data.isDemo === 1) {
            var demoResData = getDataFromStroageData(url, data, DemoConfig.demoData, DemoConfig.specialUrl, DemoConfig.demoMicAppName);
            // 如果缓存有数据而且是需要缓存的数据,直接往上抛，否则还是走网络请求
            if (demoResData && demoResData.cacheKey !== "noCache") {
                var retData = demoResData.resData;
                // 重要:
                // 如果连code都没有，说明是缓存中没有数据，需要toast提示，方便调试检查没有缓存到该缓存的数据
                // 上线时建议干掉这句showToast
                if (!Object.prototype.hasOwnProperty.call(retData, 'code')) {
                    // showToast("该缓存的数据没缓存: " + url);
                }
                console.log("Net res ," + url + ",", Number(new Date()));
                // res(retData);
                setTimeout(function () {
                    console.log("Net res timeout," + url + ",", Number(new Date()));
                    res(retData);
                }, 350);
                return;
            }
        }
        // end mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理

        encryptData(data).then(function (encryptData) {
            // 加密request数据
            cryptKey = encryptData.key;
            zip = encryptData.zip;
            keyType = encryptData.keyType;
            // 兼容mPaaS网关请求处理， 如果是mPaaS浏览器，直接走mPaaS网关请求
            if (window.AlipayJSBridge) {
                console.log("doPostRequest -- window.AlipayJSBridge...");
                return aliCallRequest(url, encryptData);
            }
            console.log("doPostRequest -- axios request...");
            // 非mPaaS环境，走原来的http请求
            return axios.request(url, {
                method: 'post',
                data: encryptData
            });
        }).then(function (response) {
            // 服务器response处理
            return decryptData(response, cryptKey, zip, keyType); // 去解密
        }).then(function (decryptData) {
            // 解密成功(包含code==0和code！=0，由业务自己处理code)
            // start mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
            stroageData(url, data, decryptData, DemoConfig.isCatchData, DemoConfig.specialUrl, DemoConfig.demoMicAppName);
            // end mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
            res(decryptData);
        })["catch"](function (e) {
            // 统一处理异常情况, 由调用层自己处理异常情况
            console.log("doPostRequest e = " + JSON.stringify(e));
            rej(e);
        });
    });
    return p;
}

/**
 * 加密数据
 */
function encryptData() {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var p = new Promise(function (res, rej) {
        EncryptionFunction(data).then(function (eData) {
            var d = {
                bdata: JSON.stringify({
                    data: eData.encryptData,
                    zip: eData.zip,
                    key: eData.key,
                    keyType: eData.keyType
                }),
                key: eData.key,
                zip: eData.zip,
                keyType: eData.keyType
            };
            //          DecryptionFunction(JSON.parse(d.bdata)).then(function (result) {
            //              console.log("加密后："+JSON.stringify(result));
            //          });
            res(d);
        });
    });
    return p;
}

/**
 * 解密数据
 */
function decryptData(result, key, zip, keyType) {
    var p = new Promise(function (res, rej) {
        var response = result;
        if ('string' === typeof result) {
            response = JSON.parse(result);
        }
        // ssp服务的response最外层协议结构
        // mj 2019-04-17 协议新增ecode，emsg; 用于拼接在desc后面展示给用户
        var ret = {
            code: response.code,
            ecode: response.ecode,
            emsg: response.emsg,
            desc: response.rdesc,
            data: response.data
        };
        if (isSuccess(response) && !!response.data) {
            // code == 0并且data不能为空
            var d = {
                zip: zip,
                key: key,
                keyType: keyType,
                data: response.data
            };
            DecryptionFunction(d).then(function (result) {
                ret.data = result;
                res(ret);
            });
        } else {
            // 服务器返回异常code
            res(ret);
        }
    });
    return p;
}

/**
 * result 是否成功 code == 0
 * @param result
 * @returns {boolean}
 */
export function isSuccess(result) {
    if (result && result.code === 0) {
        return true;
    } else {
        return false;
    }
}

//AliCall 请求统一处理 20190307 zw
function aliCallRequest(url, data) {
    return new Promise(function (res, rej) {
        // 通过url解析mPaaS网关api名称、path参数
        var mgwInfo = parseMGRParam(url, "attendance");

        // 补充mPaaS配置参数
        data["apiName"] = mgwInfo.apiName; // 接口名称 xxx.do,mPaaS网关api上的path参数
        data["cryptPostFlag"] = true; // 数据加密

        var params = [];
        params.push(data);
        console.log('003.params:', params);
        AlipayJSBridge.call('rpc', {
            operationType: mgwInfo.apiOptType, // mPaaS网关api名称
            requestData: params, // 业务参数
            headers: {
                'content-type': 'application/x-www-form-urlencoded;'
            },
            timeout: 90
        }, function (result) {
            console.log('002 mgwInfo: ' + JSON.stringify(mgwInfo) + ', result- :' + JSON.stringify(result));
            // rpc 本身异常
            if (result.errorMessage) {
                // Toast(result.errorMessage || '请求失败 ali rpc error');
                var _errInfo = {
                    errorCode: result.error,
                    errorDesc: result.errorMessage + "[" + result.error + "]"
                };
                rej(_errInfo);
            } else {
                // 不是rpc异常，就处理业务返回码
                res(result);
            }
        });
    })["catch"](function (error) {
        console.log("aliCallRequest Error = " + JSON.stringify(error));
        rej(errInfo);
    });
}

/**
 * @description: 从URL解析网关请求的参数
 * @param url：请求地址; appServer 应用服务名称（如 approval）
 * @return:
 * @Author: miaoju
 * @Date: 2019-06-26 00:19:25
 */
function parseMGRParam(url, appServer) {
    var ret = {
        apiName: "", // 接口名称 ： home/bankoutlets/downloadBankList.do
        apiOptType: "" // 网关配置的接口 attendanceAction
    };
    if (url) {
        // 1、解析mPaaS网关api的operationType
        var appName = parseMGWOptType(url, appServer);
        ret.apiOptType = appName + "Action";
        //2、解析api接口参数 apiName
        ret.apiName = parseApiName(url, appName);
    }
    return ret;
}

/**
 * 解析swp的请求url，路径里面都是带home的，取url中home/后面的一段地址作为 apiName
 * url例子： https://yqtdev.sinosun.com:18094/yqt/bankOutlets/home/bankoutlets/downloadBankList.do
 * 返回：home/bankoutlets/downloadBankList.do
 */
function parseApiName(url, appName) {
    var apiName = url;
    var micAppName = appName + "/";
    if (url) {
        var pos = url.indexOf(micAppName);
        if (pos > -1) {
            // 取url中home/后面的一段地址作为 apiName
            apiName = url.substr(pos + micAppName.length);
        }
    }
    return apiName;
}

/**
 * 解析url,获取mPaaS网关的api的operationType, 服务名+Action
 * url例子： https://yqtdev.sinosun.com:18094/yqt/bankOutlets/home/bankoutlets/downloadBankList.do
 * 返回：bankOutletsAction
 */
function parseMGWOptType(url, appServer) {
    var apiOptType = appServer;
    // 如果url包含了/yqt/，后面就是服务名称
    if (url && url.indexOf("/yqt/") > -1) {
        var subUrl = url.substr(url.indexOf("/yqt/") + "/yqt/".length);
        apiOptType = subUrl.substr(0, subUrl.indexOf("/"));
    }
    return apiOptType;
}

/*
 * 生成不重复请求seq
*/
function newSeq() {
    var seqGUID = new GUID(); // guid用于生成不重复请求seq
    var seq = Math.round(Number(new Date())) + "" + Math.round(Math.random(seqGUID.newGUID()) * 1000);
    return seq;
}

/**
 * 处理服务器响应结果, 主要是处理错误码提示
 * @param {} result
 */
function handleResult(result) {
    // 如果code非0，表示业务失败，转换错误提示，并根据调用者showError决定是否自动提示toast
    if (!isSuccess(result)) {
        var localDesc = '';
        // emsg 为服务器接口返回的可直接展示用户的 msg描述, 如果不为空，优先展示emsg，否则从本地取desc
        if (!!result.emsg) {
            localDesc = result.emsg;
        } else {
            result.desc = transferErrorDesc(result);
            localDesc = result.desc;
        }
        // 错误描述追加显示 ecode， 如： "账号不存在[21111003]"
        if (!!result.ecode) {
            localDesc += "[" + result.ecode + "]";
        }
        // 本地拼接的用于显示给用户的edesc："账号不存在[21111003]"， 如果界面需要自己提示toast，可以用这个edesc字段
        result.edesc = localDesc;
    }
    return result;
}

/**
 * 获取用户信息
 * @returns {Promise<any>}
 * @constructor
 */
// 当前从native获取的用户信息
export var nativeInfo = {};
function getUserInfo() {
    var p = new Promise(function (res, rej) {
        // 如果内存中已经取到过uaid，不在从app获取
        if (nativeInfo.UAId) {
            res(nativeInfo);
        } else {
            // 从app获取用户信息
            return GetUserInfoFunction({
                isNeedSeesion: true
            }).then(function (uaData) {
                if (uaData) {
                    //如果native登录
                    nativeInfo = uaData;
                    nativeInfo.UAName = uaData.uName;
                    // begin 添加登录校验lgparam
                    var lgParam = {};
                    try {
                        lgParam = JSON.parse(uaData.lgParam);
                    } catch (e) {
                        // 问题：PC端给的值与app不一致，后续要pc端修改一下
                        lgParam = { session: uaData.lgParam };
                    }
                    nativeInfo.lgParam = lgParam;
                    // end 添加登录校验lgparam

                    if (uaData.deviceId) {
                        nativeInfo.deviceId = uaData.deviceId;
                    }
                    if (uaData.token) {
                        nativeInfo.token = JSON.parse(uaData.token);
                    }
                    res(nativeInfo);
                } else {
                    var _errInfo2 = {
                        errorCode: "100003",
                        errorDesc: "用户信息获取失败[100003]"
                    };
                    console.log("GetUserInfo - getuser info from native failed...");
                    rej(_errInfo2);
                }
            });
        }
    });
    return p;
}

/**
 * 检查网络状态
 * @constructor
 */
function checkNetWrok() {
    var p = new Promise(function (res, rej) {
        CheckNetWorkFunction().then(function (statusData) {
            //获取网络状态
            if (statusData.contectState) {
                res(statusData);
            } else {
                var _errInfo3 = {
                    errorCode: "100004",
                    errorDesc: "网络连接不可用[100004]"
                };
                rej(_errInfo3);
            }
        });
    });
    return p;
}

/**
 * 兼容SWP的webserver后端代码
 * @param {Object} result
 */
function compatiableData(result) {
    if (!!result.data) {
        result.data["code"] = result.code;
        return result.data;
    }
    return result;
}

/**
 * 将server的错误码转换为用户提示信息
 * @param result
 * @returns {string}
 */
function transferErrorDesc(result) {
    if (result && result.code) {
        var desc = ERROR_MAP[result.code] || result.desc;
        result.desc = desc;
    }
    return result.desc;
}

/**
 * post请求，不携带uaid、cpyId
 * @param url
 * @param data
 */
export function doPostWithoutUser(url) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var reqUrl = urlhost + url;
    var p = new Promise(function (res, rej) {
        checkNetWrok().then(function () {
            // 检查网络
            return doPostRequest(reqUrl, data); // 调用协议层网络请求
        }).then(function (result) {
            // server端返回数据，不管code是否为0，都走这里统一处理
            result = handleResult(result);
            // 最后将数据返回调用者
            // res(compatiableData(result));
            res(result);
        })["catch"](function (e) {
            // 统一处理异常情况, 并且rej给调用者，界面需要处理rej情况
            rej(e);
        });
    });
    return p;
}

/**
 * post请求(data加密)
 * @param url
 * @param data
 */
export function doPost(reqUrl) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var p = new Promise(function (res, rej) {
        checkNetWrok().then(function () {
            // 检查网络
            return getUserInfo(data); // 从app获取用户信息
        }).then(function (result) {
            if (!data.UAId) {
                data.UAId = result.UAId; // 填充用户信息, TODO 后续的session可以在这里添加
            }
            if (!data.cpyId) {
                data.cpyId = result.cpyId;
            }
            data.isDemo = result.isDemo;

            try {
                data.lgParam = JSON.parse(JSON.stringify(result.lgParam)); // 登录校验参数,从app登录返回获取来的
                data.lgParam.seq = newSeq(); // 请求seq不能重复
            } catch (e) {
                console.log('从app获取session失败');
            }
            //传出去的对象解除引用
            return doPostRequest(reqUrl, JSON.parse(JSON.stringify(data))); // 调用协议层网络请求
        }).then(function (result) {
            // server端返回数据，不管code是否为0，都走这里统一处理
            // 如果code非0，表示业务失败，转换错误提示，并根据调用者showError决定是否自动提示toast
            result = handleResult(result);
            // 最后将数据返回调用者
            res(result);
        })["catch"](function (e) {
            // 统一处理异常情况, 并且rej给调用者，界面需要处理rej情况
            rej(e);
        });
    });
    return p;
}

export function setDemoConfig(config) {
    DemoConfig.demoMicAppName = config.demoMicAppName;
    DemoConfig.noCaheList = config.noCaheList;
    DemoConfig.specialUrl = config.specialUrl;
    DemoConfig.isCatchData = config.isCatchData;
    DemoConfig.demoData = config.demoData;
}