'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _popup = require('../mixins/popup');

var _button = require('../button');

var _button2 = _interopRequireDefault(_button);

var _classnames = require('../_utils/classnames');

var _classnames2 = _interopRequireDefault(_classnames);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var Modal = {
    mixins: [_popup.PopupMixin],

    props: {
        title: String,
        width: [Number, String],
        message: String,
        className: null,
        callback: Function,
        beforeClose: Function,
        messageAlign: String,
        cancelButtonText: {
            type: String,
            'default': '取消'
        },
        cancelButtonColor: String,
        confirmButtonText: {
            type: String,
            'default': '确认'
        },
        confirmButtonColor: String,
        showCancelButton: Boolean,
        transition: {
            type: String,
            'default': 'sn-modal-bounce'
        },
        showConfirmButton: {
            type: Boolean,
            'default': true
        },
        overlay: {
            type: Boolean,
            'default': true
        },
        closeOnClickOverlay: {
            type: Boolean,
            'default': false
        }
    },

    data: function data() {
        return {
            loading: {
                confirm: false,
                cancel: false
            }
        };
    },


    methods: {
        onClickOverlay: function onClickOverlay() {
            this.handleAction('overlay');
        },
        handleAction: function handleAction(action) {
            var _this = this;

            this.$emit(action);

            if (this.beforeClose) {
                this.loading[action] = true;
                this.beforeClose(action, function (state) {
                    if (state !== false && _this.loading[action]) {
                        _this.onClose(action);
                    }

                    _this.loading.confirm = false;
                    _this.loading.cancel = false;
                });
            } else {
                this.onClose(action);
            }
        },
        onClose: function onClose(action) {
            this.close();

            if (this.callback) {
                this.callback(action);
            }
        },
        onOpened: function onOpened() {
            this.$emit('opened');
        },
        onClosed: function onClosed() {
            this.$emit('closed');
        }
    },

    render: function render() {
        var _this2 = this;

        var h = arguments[0];

        if (!this.shouldRender) {
            return;
        }

        var message = this.message,
            messageAlign = this.messageAlign,
            confirmButtonText = this.confirmButtonText,
            $slots = this.$slots;

        var messageSlot = $slots['default'];
        var title = $slots['title'] || this.title;

        var Title = title && h(
            'div',
            { 'class': {
                    'sn-modal-header': true,
                    isolated: !message && !messageSlot
                } },
            [title]
        );

        var Content = (messageSlot || message) && h(
            'div',
            { 'class': 'sn-modal-content' },
            [messageSlot || h('div', {
                domProps: {
                    'innerHTML': message
                },

                'class': (0, _classnames2['default'])('sn-modal-message', 'sn-modal-message-' + messageAlign)
            })]
        );

        var hasButtons = this.showCancelButton && this.showConfirmButton;
        var ButtonGroup = h(
            'div',
            { 'class': {
                    'sn-modal-footer': true,
                    'sn-modal-footer-button-group': hasButtons
                } },
            [this.showCancelButton && h(
                _button2['default'],
                {
                    attrs: {
                        shape: 'square',

                        loading: this.loading.cancel
                    },
                    'class': 'sn-modal-footer-cancel', style: { color: this.cancelButtonColor },
                    on: {
                        'click': function click() {
                            return _this2.handleAction('cancel');
                        }
                    }
                },
                [this.cancelButtonText]
            ), this.showConfirmButton && h(
                _button2['default'],
                {
                    attrs: {
                        shape: 'square',

                        loading: this.loading.confirm
                    },
                    'class': 'sn-modal-footer-confirm', style: { color: this.confirmButtonColor },
                    on: {
                        'click': function click() {
                            return _this2.handleAction('confirm');
                        }
                    }
                },
                [confirmButtonText]
            )]
        );

        return h(
            'transition',
            {
                attrs: {
                    name: this.transition
                },
                on: {
                    'afterEnter': this.onOpened,
                    'afterLeave': this.onClosed
                }
            },
            [h(
                'div',
                {
                    directives: [{
                        name: 'show',
                        value: this.value
                    }],
                    attrs: {
                        role: 'modal'
                    },
                    'class': (0, _classnames2['default'])('sn-modal', this.className),
                    style: { width: this.width }
                },
                [Title, Content, ButtonGroup]
            )]
        );
    }
};

exports['default'] = Modal;
module.exports = exports.default;