'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.stroageData = stroageData;
exports.getDataFromStroageData = getDataFromStroageData;

var _jsBase = require('js-base64');

// 保存数据的key前缀，用于在导出数据时，能够准确的导出本应用的数据
var demoAppKeyFlag = 'demo_';

/**
 * 根据请求生成缓存的key
 * url 请求url
 * params  请求参数，有部分请求需要从参数里面取一些信息加入到key里面
 */
/**
 * demo缓存工具类
 */
function generateStroageKey(url, params, specialUrl, demoMicAppName) {
    // 不需要缓存的url请求，直接返回 noCache
    if (isNoCacheUrl(url, params)) {
        return "noCache";
    }
    // url 解析成key
    var key = paseUrl2Key(url);
    var stroageKey = specialUrl(url, params, key);
    console.log("generateStroageKey -- url:" + url + ", params:" + JSON.stringify(params) + ", stroageKey : " + stroageKey);
    return demoAppKeyFlag + demoMicAppName + '_' + stroageKey;
}

/**
 *  把url解析成key
 * @param {string} [url='']
 */
function paseUrl2Key() {
    var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    // TODO 解析方法待优化
    // https://yqtdev.sinosun.com:18094/yqt/bankOutlets/home/bankoutlets/downloadBankList.do
    // 取 /yqt -- .do 中间的部分作为key
    var reqUrl = url;
    var subUrl = url;
    // 去掉 "/yqt/"
    if (reqUrl && reqUrl.indexOf("/yqt/") > -1) {
        subUrl = reqUrl.substr(reqUrl.indexOf("/yqt/") + "/yqt/".length);
    }
    // 去掉最后的 ".do"
    if (subUrl.indexOf(".do") > -1) {
        subUrl = subUrl.substr(0, subUrl.length - ".do".length);
    }
    var key = subUrl;
    // 所有'/' -> '_'
    if (subUrl) {
        var reg = new RegExp('/', "g");
        key = subUrl.replace(reg, '_');
    }
    return key;
}

/**
 * 是否不需要缓存的请求
 * @param {*} url
 * @param {*} data
 */
function isNoCacheUrl(url, params) {
    var cacheFlag = false;
    if (url && noCaheList) {
        // 从不缓存列表中查找
        cacheFlag = noCaheList.some(function (element) {
            return url.indexOf(element) > -1;
        });
    }
    return cacheFlag;
}

/**
 * 缓存数据
 * @export
 * @param {*} url 请求url
 * @param {*} reqData  请求参数
 * @param {*} resData  服务器返回数据（解密后的）
 */
function stroageData(url, reqData, resData, isCatchData, specialUrl, demoMicAppName) {
    // 非抓数据模式， 直接return
    if (!isCatchData) {
        return;
    }
    // 如果对应url不需要缓存数据，stroageKey返回 "noCache"
    var stroageKey = generateStroageKey(url, reqData, specialUrl, demoMicAppName);
    console.log("stroageData -- stroageKey:" + stroageKey + ", data:" + JSON.stringify(resData));
    // 不需要缓存，直接return
    if (!stroageKey || stroageKey === 'noCache') {
        return;
    }

    // 缓存数据到localStorage,
    if (stroageKey && resData) {
        var baseData = _jsBase.Base64.encode(JSON.stringify(resData));
        var storage = window.localStorage;
        storage.setItem(stroageKey, baseData);
    }
}

/**
 * 从缓存js中读取demo数据
 * @param {*} url 请求url
 * @param {*} reqData 请求参数
 */
function getDataFromStroageData(url, reqData, demoData, specialUrl, demoMicAppName) {
    // 如果对应url不需要缓存数据，stroageKey返回 "noCache"
    var stroageKey = generateStroageKey(url, reqData, specialUrl, demoMicAppName);
    var retObj = {};
    // 不需要缓存，直接return
    if (!stroageKey || stroageKey === 'noCache') {
        retObj['cacheKey'] = 'noCache';
        return retObj;
    }

    retObj['cacheKey'] = stroageKey;
    retObj['resData'] = getDataFromCache(stroageKey, demoData);

    // 从demo缓存获取数据,
    return retObj;
}

/**
 *从demo缓存获取数据
 *
 * @param {string} [stroageKey='']
 */
function getDataFromCache() {
    var stroageKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var demoData = arguments[1];

    // 从mock数据中获取缓存数据
    var resData = demoData[stroageKey];
    var resObj = {};
    try {
        if (resData) {
            // base64解密，转对象
            resData = _jsBase.Base64.decode(resData);
            resObj = JSON.parse(resData);
        }
    } catch (e) {
        console.log("getDataFromCache - exception : " + e);
    }

    return resObj;
}

/**
 * 工具方法，导出数据
 */
window.exportDemoData = function () {
    // 需要导出的数据
    var exportData = getAllYqtDemoDataFromStroage();
    return exportData;
};

/**
 * 获取H5所有需要迁移的storage缓存
 */
function getAllYqtDemoDataFromStroage() {
    var storage = window.localStorage;
    var cacheDatasJson = {};
    if (storage) {
        for (var i = 0; i < storage.length; ++i) {
            var key = storage.key(i);
            // 是否yqtdemo数据
            if (key.indexOf(demoAppKeyFlag) > -1) {
                // 将需要迁移的key-value存入json数组
                var value = storage.getItem(key);
                cacheDatasJson[key] = value;
            }
        }
        var transferData = JSON.stringify(cacheDatasJson);
        console.log("getAllYqtDemoDataFromStroage - yqtDemo Data : " + transferData);
        return transferData;
    } else {
        console.log("getAllYqtDemoDataFromStroage - exception storage is null...");
    }
    return "";
}