'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _ajaxUploader = require('./ajax-uploader');

var _ajaxUploader2 = _interopRequireDefault(_ajaxUploader);

var _uploadList = require('./upload-list');

var _uploadList2 = _interopRequireDefault(_uploadList);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

exports['default'] = {
    name: "UpLoad",
    prefixCls: 'sn-upload',
    props: {
        readOnly: {
            type: Boolean,
            'default': false
        },
        listType: {
            type: String,
            'default': function _default() {
                return 'text';
            }
        }, // or pictrue
        disabled: {
            type: Boolean,
            'default': function _default() {
                return false;
            }
        },
        beforeUpload: Function,
        showUploadList: {
            type: Boolean,
            'default': function _default() {
                return true;
            }
        },
        action: {
            type: String,
            'default': function _default() {
                return '';
            }
        },
        customRequest: Function,
        defaultFileList: {
            type: Array,
            'default': function _default() {
                return [];
            }
        }
    },
    data: function data() {
        return {
            fileList: [].concat(this.defaultFileList)
        };
    },

    methods: {
        onClick: function onClick() {
            if (this.$refs.ajaxUpload) {
                this.$refs.ajaxUpload.onClick();
            }
        },
        start: function start(file) {
            file.status = 'uploading';
            this.fileList.push(file);
            this.$emit('onStart', file, this.fileList);
        },
        error: function error(info) {
            console.log('onError', info);
            this.onChange(info);
        },
        progress: function progress(info, file) {
            this.$emit('onProgress', info, file);
        },
        success: function success(file, result, xhr) {
            this.$emit('onSuccess', file, result);
        },
        preview: function preview(file) {
            this.$emit('onPreview', file);
        },
        remove: function remove(file, index) {
            this.fileList.splice(index, 1);
            this.$emit('onRemove', file, index);
        },
        reBeforeUpload: function reBeforeUpload(file, files) {
            if (!this.beforeUpload) {
                return true;
            }
            var result = this.beforeUpload(file, files);
            if (result === false) {
                return false;
            } else if (result && result.then) {
                return result;
            }
            return true;
        },
        renderUploadList: function renderUploadList() {
            var h = this.$createElement;
            var _$props = this.$props,
                showUploadList = _$props.showUploadList,
                listType = _$props.listType;

            var uploadListProps = {
                props: {
                    items: this.fileList,
                    listType: listType
                },
                on: {
                    remove: this.remove,
                    preview: this.preview
                }
            };
            if (!showUploadList) {
                return null;
            } else {
                return h(_uploadList2['default'], uploadListProps);
            }
        }
    },
    render: function render() {
        var h = arguments[0];

        var children = this.$slots['default'];
        var listType = this.$props.listType;

        var AjaxUploaderProps = {
            ref: "ajaxUpload",
            props: (0, _extends3['default'])({}, this.$props, {
                beforeUpload: this.reBeforeUpload
            }),
            on: {
                start: this.start,
                error: this.error,
                progress: this.progress,
                success: this.success
            }
        };

        var uploadList = this.renderUploadList();

        var uploadButton = this.readOnly ? null : h(
            _ajaxUploader2['default'],
            AjaxUploaderProps,
            [' ', children]
        );

        var conMaps = ['upload', 'list'];
        var list = function list() {
            return conMaps.map(function (type, index) {
                if (type === 'list') {
                    return { uploadList: uploadList };
                } else {
                    return { uploadButton: uploadButton };
                }
            });
        };
        // if (listType === 'picture') {
        //     return (
        //         <transition-group>
        //             {uploadList}
        //             {uploadButton}
        //         </transition-group>
        //     );
        // }
        return h(
            'div',
            { 'class': 'sn-upload' },
            [uploadButton, uploadList]
        );
    }
};
module.exports = exports.default;